#include "mex.h"
#include "matrix.h"
#include <math.h>

/* Global inverted permutation (for comparator function) */
unsigned int* g_inversePermutation;

/* Int comparator that takes into account topic ordering */
int permutationCompare(const void *arg1, const void *arg2)
{
	unsigned int* num1 = (unsigned int*)arg1;
	unsigned int* num2 = (unsigned int*)arg2;
	if (g_inversePermutation[*num1] < g_inversePermutation[*num2])
	{
		return -1;
	}
	else if (g_inversePermutation[*num1] > g_inversePermutation[*num2])
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/* Inverts permutation into dest */
void invertPermutation(unsigned int* dest, unsigned int* src, size_t length)
{
	unsigned int i;
	for (i = 0; i < length; ++i)
	{
		dest[src[i]] = i;
	}
}

/* Converts v vector to permutation; length refers to true length of permutation, which is one greater than length of V */
void vToPi(unsigned int* dest, unsigned int* v, size_t length)
{
	int i, j;
	dest[0] = (unsigned int)length - 1;
	for (i = (int)length - 2; i >= 0; --i)
	{
		/* Insert into permutation */
		for (j = (int)length - i - 1; j >= (int)v[i] + 1; --j)
		{
			dest[j] = dest[j - 1];
		}
		dest[v[i]] = i;
	}
}

/* Sets global permutation for comparator */
void setGlobalPermutation(unsigned int* v)
{
	vToPi(g_proposedOrdering, v, g_K);
	invertPermutation(g_inversePermutation, g_proposedOrdering, g_K);
}