#include "mex.h"
#include "matrix.h"

void resample_rho(double* totalVParam, double* nminusjParam, double* v_0Param, double* lastRhoParam, mxArray* sample_rhoParams[])
{
	unsigned int i, j;
	int status;
	mxArray* rhoArray;

	/* Resample each Mallows parameter rho */
	for (i = 0; i < g_K - 1; ++i)
	{
		*totalVParam = g_nvs[i];
		*nminusjParam = (double)g_K - (double)i;
		*v_0Param = g_v_0[i];
		*lastRhoParam = g_rho[i];
		/* Retry up to 10 times */
		for (j = 0; j < 10; ++j)
		{
			status = mexCallMATLAB(1, &rhoArray, 7, sample_rhoParams, "sample_rho");
			if (status == 0)
			{
				break;
			}
			mexPrintf("Warning: slice sampling failed on rho[%d]\n", i);
		}
		if (j == 10)
		{
			mexErrMsgTxt("Could not slice resample rho\n");
		}

		g_rho[i] = mxGetScalar(rhoArray);
		/* mexPrintf("rho[%d]: %1.6f\n", i, rho[i]); */

		mxDestroyArray(rhoArray);
	}
}