#include "mex.h"
#include "matrix.h"

void resample_t(unsigned int document, double* order)
{
	unsigned int j, k;
	unsigned int sentIdx;
	unsigned int topic;
	unsigned int numSents = g_numSents[document];

	/* Set global permutation */
	setGlobalPermutation(g_v + index2d(0, g_K - 1, document));

	/* For each sentence, consider switching topic */
	for (j = 0; j < g_numSents[document]; ++j)
	{
		sentIdx = round(order[j] - 1.0);

		/* Remove this current sentence from ntau */
		topic = g_t[index2d(sentIdx, g_maxNumSents, document)];
		--g_ntau[index2d(topic, g_K, 0)];
		--g_ntaus[index2d(0, 1, 0)];

		/* Consider each possible switch */
		for (k = 0; k < g_K; ++k)
		{
			/* Perform proposed topic change */
			memcpy(g_proposedTopics, g_t + index2d(0, g_maxNumSents, document), numSents * sizeof(unsigned int));
			g_proposedTopics[sentIdx] = k;
			qsort(g_proposedTopics, numSents, sizeof(unsigned int), permutationCompare);

			/* Prior of new topic set + likelihood */
			g_topicProb[k] = log(((double)g_ntau[index2d(k, g_K, 0)] + g_tau_0)
				/ ((double)g_ntaus[index2d(0, 1, 0)] + g_tau_0 * (double)g_K))
				+ sentenceProb(document, 0, numSents, g_proposedTopics, g_documentWords);
		}

		/* Sample from normalized topic probs */
		topic = sampleNormalized(g_topicProb, g_K, g_unifrand[g_randIdx++]);
		g_t[index2d(sentIdx, g_maxNumSents, document)] = topic;

		/* Add new topic selection back to ntau */
		++g_ntau[index2d(topic, g_K, 0)];
		++g_ntaus[index2d(0, 1, 0)];
	}
}