#include "mex.h"
#include "matrix.h"
#include <math.h>

/* Computes and caches log of Multinomial-compound-Dirichlet probability of single sentence or set of sentences in a document */
double sentenceProb(unsigned int document, unsigned int start, unsigned int length,
					unsigned int* topics, unsigned int* documentWords)
{
	unsigned int numSents = g_numSents[document];

	mxArray* words = mxGetFieldByNumber(g_s, index2d(2, 3, document), 0);
	mwIndex* ir = mxGetIr(words);
	mwIndex* jc = mxGetJc(words);
	double* pr = mxGetPr(words);

	double result = 0.0;
	unsigned int i = start;
	unsigned int j, k, l;
	unsigned int spanLength;
	double cacheValue;
	unsigned int documentWordss;
	size_t loc, loc2;

	while (i < start + length)
	{
		/* Find length of span */
		for (spanLength = 1; i + spanLength < start + length && topics[i + spanLength] == topics[i]; ++spanLength) {}

		/* Check availability in cache */
		cacheValue = g_sentProbCache[cacheIndex(i, spanLength, numSents, topics[i])];
		if (cacheValue == 0.0)
		{
			/* Cache miss, compute value */

			/* Running counts of topic words for current topic */
			memset(documentWords, 0, g_vocabSize * sizeof(unsigned int));
			documentWordss = 0;

			for (j = 0; j < spanLength; ++j)
			{
				for (k = (unsigned int)jc[i + j]; k < jc[i + j + 1]; ++k)
				{
					for (l = 0; l < pr[k]; ++l)
					{
						loc = index2d(ir[k], g_vocabSize, topics[i]);
						loc2 = index2d(0, 1, topics[i]);
						cacheValue += log(((double)g_ntheta_t[loc] + (double)documentWords[ir[k]] + g_theta_t_0) /
							((double)g_ntheta_ts[loc2] + (double)documentWordss + g_theta_t_0 * (double)g_vocabSize));
						/* Add to temporary word count cache */
						++documentWords[ir[k]];
						++documentWordss;
					}
				}

				/* Save cached value up to this point */
				g_sentProbCache[cacheIndex(i, j + 1, numSents, topics[i])] = cacheValue;
			}
		}
		i += spanLength;
		/* Use cached value */
		result += cacheValue;
	}
	return result;
}
